# Copyright (C) 2008, 2009 by Intevation GmbH
#
# Based on code from
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/456195 which
# is Copyright (C) 2006 by Alessandro Budai
#
# This software may be used and distributed according to the terms
# of the Python License 2.3 or newer, see http://www.python.org/license

"""
Variant of httplib.HTTPConnection that connects through a proxy using
the http CONNECT method (that can be used to proxy SSL connections).
"""

import urlparse
import urllib
import httplib
import socket


default_ports = {'http' : 80, 'https' : 443}


def parse_netloc(scheme, netloc):
    """Parses netloc and returns (hostname, port).

    If the netloc string doesn't contain a port, the default port of
    scheme (either 'http' or 'https') is used.  If the port cannot be
    determined, a ValueError is raise.
    """
    host, port = urllib.splitport(netloc)
    if port is None:
        port = default_ports.get(scheme)
        if port is None:
            raise ValueError("Cannot determine port for URL scheme %r"
                             % scheme)
    else:
        port = int(port)
    return host, port


class HTTPSProxyConnection(httplib.HTTPConnection):

    def putrequest(self, method, url, skip_host=0, skip_accept_encoding=0):
        if not skip_host:
            raise ValueError("HTTPSProxyConnection.putrequest must be called"
                             " with skip_host=True, because the correct Host"
                             " request header field will not be generated"
                             " automatically")

        scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
        if not netloc or not scheme:
            raise ValueError("HTTPSProxyConnection.putrequest must be called"
                             " with a full url including a scheme and a"
                             " hostname.")

        self._real_address = parse_netloc(scheme, netloc)

        # Make sure the actual request url only contains the abspath
        # part of the URL, and not the scheme and hostname.  HTTP/1.1
        # servers are supposed to accept such URLs in requests, but
        # clients should not send them.
        url = urlparse.urlunsplit(("", "", path, query, fragment))

        httplib.HTTPConnection.putrequest(self, method, url,
                                          skip_host=skip_host,
                                     skip_accept_encoding=skip_accept_encoding)

    def connect(self):
        httplib.HTTPConnection.connect(self)
        self.send("CONNECT %s:%d HTTP/1.0\r\n\r\n" % self._real_address)
        #expect a HTTP/1.0 200 Connection established
        response = self.response_class(self.sock, strict=self.strict,
                                       method=self._method)
        (version, code, message) = response._read_status()
        #probably here we can handle auth requests...
        if code != 200:
            #proxy returned and error, abort connection, and raise exception
            self.close()
            raise socket.error, ("Proxy connection failed: %d %s"
                                 % (code, message.strip()))
        #eat up header block from proxy....
        while True:
            #should not use directly fp probably
            line = response.fp.readline()
            if line == '\r\n':
                break

        ssl = socket.ssl(self.sock, None, None)
        self.sock = httplib.FakeSocket(self.sock, ssl)
