# Copyright (C) 2009 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Tests for inteproxy.transcoder.parse_inteproxy_url"""

import unittest

import support

from inteproxy.config import parse_inteproxy_url


class TestParseInteProxyUrl(unittest.TestCase):

    rule_attributes = ["scheme", "host", "port", "path",
                       ("query", ""), ("fragment", ""),
                       "user", "password"]

    def check_parse_result(self, url, **expected):
        rule = parse_inteproxy_url(url)
        for desc in self.rule_attributes:
            if isinstance(desc, tuple):
                attribute, default = desc
            else:
                attribute = desc
                default = None
            self.assertEquals(getattr(rule, attribute),
                              expected.get(attribute, default))

    def test_owsproxy_no_user_no_port_no_query_no_fragment(self):
        self.check_parse_result("owsproxy://example.com/a/directory",
                                scheme="owsproxy", host="example.com",
                                path="/a/directory")

    def test_owsproxy_no_user_no_port_query_fragment(self):
        self.check_parse_result("owsproxy://localhost/wms?method=get#baz",
                                scheme="owsproxy", host="localhost",
                                path="/wms", query="method=get", fragment="baz")

    def test_owsproxy_no_user_port_no_query_no_fragment(self):
        self.check_parse_result("owsproxy://localhost:123/wms",
                                scheme="owsproxy", host="localhost", port=123,
                                path="/wms")

    def test_owsproxy_user_no_passwd_port_no_query_no_fragment(self):
        self.check_parse_result("owsproxy://bob@localhost:123/wms",
                                scheme="owsproxy", host="localhost", port=123,
                                path="/wms", user="bob")

    def test_owsproxy_user_passwd_port_no_query_no_fragment(self):
        self.check_parse_result("owsproxy://bob:tomato@localhost:123/my/wms",
                                scheme="owsproxy", host="localhost", port=123,
                                path="/my/wms", user="bob", password="tomato")

    def test_owsproxy_quoteduser_quotedpasswd_port_no_query_no_fragment(self):
        self.check_parse_result("owsproxy://bob%40example.com:tom%25to"
                                "@localhost:1024/wms",
                                scheme="owsproxy", host="localhost", port=1024,
                                path="/wms",
                                user="bob@example.com", password="tom%to")
